@@ -1,8 +1,8 @@
 # PageBuild.pm - Interpret MiniVend tags
 # 
-# $Id: PageBuild.pm,v 1.6 1999/06/07 08:07:23 mike Exp mike $
+# $Id: PageBuild.pm,v 1.2 1997/06/17 04:22:52 mike Exp $
 #
-# Copyright 1996-1998 by Michael J. Heins <mikeh@iac.net>
+# Copyright 1996 by Michael J. Heins <mikeh@iac.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,16 +22,18 @@ package Vend::PageBuild;
 require Exporter;
 @ISA = qw(Exporter);
 
-$VERSION = substr(q$Revision: 1.6 $, 10);
+$VERSION = substr(q$Revision: 1.2 $, 10);
 
 @EXPORT = qw ( fake_html );
 
+use Carp;
 use strict;
 use Vend::Util;
 use Vend::Data;
 use Vend::Server;
 use Vend::ValidCC;
 use Vend::Interpolate;
+use Text::ParseWords;
 
 my $Level;
 
@@ -94,15 +96,15 @@ sub fake_areatarget {
 # Returns an href to call up the last page visited.
 
 sub fake_last_page {
-    fake_page(find_special_page('catalog'));
+    fake_page($Vend::Cfg->{Special}->{'catalog'});
 }
 
 # Evaluates the [...] tags.
 
 sub fake_html {
     my($html,$level) = @_;
-    my($codere) =  '[-\w_#/.]+';
-    my($coderex) = '[-\w_:#=/.]+';
+    my($codere) =  '[\w-_#/.]+';
+    my($coderex) = '[\w-_:#=/.]+';
 	
 	$Level = '';
 	if($level =~ m:^/:) {
